unit oidebug;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, Grids, oinspect, peregist;

type
  TFormDebug = class(TForm)
    TreeView1: TTreeView;
    Splitter1: TSplitter;
    PageControl1: TPageControl;
    Properties: TTabSheet;
    Events: TTabSheet;
    PropertyList: TObjectInspector;
    EventsList: TObjectInspector;
    ImageList1: TImageList;
    procedure FormShow(Sender: TObject);

    procedure TreeView1Change(Sender: TObject; Node: TTreeNode);
    procedure FormActivate(Sender: TObject);
  private
    fOldHandler : TMessageEvent;
    procedure HandleMessages (var Msg : TMsg; var Handled : boolean);
    function AddTreeNode(parent : TTreeNode; comp : TComponent) : TTreeNode;
    procedure FillTree;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    { Public declarations }
  end;

var
  FormDebug: TFormDebug;

implementation

{$R *.DFM}

{******************************************************************}

constructor TFormDebug.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  fOldHandler := Application.OnMessage;
  Application.OnMessage := HandleMessages;
end;

{------------------------------------------------------------------}

destructor TFormDebug.Destroy;
begin
  Application.OnMessage := fOldHandler;
  inherited Destroy;
end;

{------------------------------------------------------------------}

procedure TFormDebug.Notification(AComponent: TComponent; Operation: TOperation);
var i : integer;
begin
  if (Operation = opRemove) and not (csDestroying in ComponentState) then
    begin
      for i := 0 to TreeView1.Items.Count - 1 do
        if TreeView1.Items[i].Data = AComponent then
          begin
            TreeView1.Items.Delete(TreeView1.Items[i]);
            exit;
          end;
    end;
end;

{------------------------------------------------------------------}

procedure TFormDebug.HandleMessages (var Msg : TMsg; var Handled : boolean);
begin
  handled := false;
  with Msg do
    if (message = WM_KEYDOWN) and (word(wParam) = worD('Z')) and
     (KeyDataToShiftState(lParam) = [ssCtrl, ssAlt])
    then
      if Visible then
        Hide
      else
        Show;
end;

{------------------------------------------------------------------}

function TFormDebug.AddTreeNode(parent : TTreeNode; comp : TComponent) : TTReeNode;
var i : integer;
begin
  with TreeView1.Items do
    begin
      if comp.name <> '' then
        begin
          result := AddChildObject(parent, comp.name, comp);
          if parent <> nil then
            begin
              result.ImageIndex := 1;
              result.StateIndex := 1;
              result.SelectedIndex := 1;
            end;  
        end;
      for i := 0 to comp.ComponentCount - 1 do
        AddTreeNode(result, comp.Components[i]);
    end;
end;

{------------------------------------------------------------------}

procedure TFormDebug.FillTree;
var i : integer;
    node : TTreeNode;
begin
  with TreeView1.Items do
    begin
      BeginUpdate;
      Clear;
      for i := 0 to Screen.FormCount - 1 do
        if (Screen.Forms[i] <> self) and (Screen.Forms[i].Visible) then
          begin
            node := AddTreeNode(nil, Screen.Forms[i]);
            if (Screen.ActiveForm = Screen.Forms[i]) then
              TreeView1.Selected := node;
          end;
      EndUpdate;
    end;
  TreeView1.FullExpand;
end;

{------------------------------------------------------------------}

procedure TFormDebug.FormShow(Sender: TObject);
begin
  PageControl1.ActivePage := Properties;
end;

{------------------------------------------------------------------}

procedure TFormDebug.TreeView1Change(Sender: TObject; Node: TTreeNode);
begin
  PropertyList.CurrentControl := TreeView1.Selected.Data;
  EventsList.CurrentControl := TreeView1.Selected.Data;
end;

{******************************************************************}

procedure TFormDebug.FormActivate(Sender: TObject);
begin
  FillTree;
end;

end.

